// src/api.ts
import axios from "axios";

// src/Plugin/BaseUrl/BaseUrlRequestInterceptor.ts
function BaseUrlRequestInterceptor(baseUrlGetter) {
  async function resolved(requestConfig) {
    requestConfig.baseURL = baseUrlGetter();
    return requestConfig;
  }
  return {
    name: "BaseUrlRequestInterceptor",
    type: "request" /* REQUEST */,
    priority: 1,
    resolved
  };
}

// src/Error/ClientError.ts
import { AxiosError } from "axios";
var ClientError = class _ClientError extends AxiosError {
  constructor(config, request, response, message = "A client error occurred.", code = "0") {
    super(message, code, config, request, response);
    this.message = message;
    this.name = "ClientError";
    this.status = response.status;
    this.config = config;
    this.request = request;
    this.response = response;
    Object.defineProperty(this, "isAxiosError", {
      value: true,
      writable: false,
      enumerable: false,
      configurable: false
    });
    if ("captureStackTrace" in Error && typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, _ClientError);
    } else {
      this.stack = new Error(message).stack;
    }
  }
  status;
  config;
  request;
  response;
};

// src/Error/ServerError.ts
import { AxiosError as AxiosError2 } from "axios";
var ServerError = class _ServerError extends AxiosError2 {
  constructor(config, request, response, message = "A server error occurred.", code = "0") {
    super(message, code, config, request, response);
    this.message = message;
    this.name = "ServerError";
    this.status = response.status;
    this.config = config;
    this.request = request;
    this.response = response;
    Object.defineProperty(this, "isAxiosError", {
      value: true,
      writable: false,
      enumerable: false,
      configurable: false
    });
    if ("captureStackTrace" in Error && typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, _ServerError);
    } else {
      this.stack = new Error(message).stack;
    }
  }
  status;
  config;
  request;
  response;
};

// src/Error/NetworkError.ts
import { AxiosError as AxiosError3 } from "axios";
var NetworkError = class _NetworkError extends AxiosError3 {
  constructor(message = "A network error occurred.") {
    super(message);
    this.message = message;
    this.name = "NetworkError";
    Object.defineProperty(this, "isAxiosError", {
      value: true,
      writable: false,
      enumerable: false,
      configurable: false
    });
    if ("captureStackTrace" in Error && typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, _NetworkError);
    } else {
      this.stack = new Error(message).stack;
    }
  }
};

// src/Error/TimeoutError.ts
import { AxiosError as AxiosError4 } from "axios";
var TimeoutError = class _TimeoutError extends AxiosError4 {
  constructor(message = "A timeout error occurred.") {
    super(message);
    this.message = message;
    this.name = "TimeoutError";
    this.isAxiosError = true;
    if ("captureStackTrace" in Error && typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, _TimeoutError);
    } else {
      this.stack = new Error(message).stack;
    }
  }
};

// src/Error/AbortedError.ts
import { AxiosError as AxiosError5 } from "axios";
var AbortedError = class _AbortedError extends AxiosError5 {
  constructor(message = "A timeout error occurred.") {
    super(message);
    this.message = message;
    this.name = "AbortedError";
    Object.defineProperty(this, "isAxiosError", {
      value: true,
      writable: false,
      enumerable: false,
      configurable: false
    });
    if ("captureStackTrace" in Error && typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, _AbortedError);
    } else {
      this.stack = new Error(message).stack;
    }
  }
};

// src/Plugin/ErrorHandler/ErrorHandlerResponseInterceptor.ts
function ErrorHandlerResponseInterceptor() {
  function getErrorMessage(response, fallback) {
    if (response.data && response.data.message) {
      return response.data.message;
    }
    if (response.data && response.data?.detail) {
      return response.data.detail;
    }
    if (response.data && response.data?.["hydra:description"]) {
      return response.data["hydra:description"];
    }
    return fallback;
  }
  async function resolved(response) {
    if (response.status >= 400 && response.status < 500) {
      throw new ClientError(response.config, response.request, response);
    }
    if (response.status >= 500 && response.status < 600) {
      throw new ServerError(response.config, response.request, response);
    }
    return response;
  }
  async function rejected(error) {
    if (!error?.config) {
      throw error;
    }
    if (error?.code === "ETIMEDOUT") {
      throw new TimeoutError();
    }
    if (error?.code === "ECONNABORTED") {
      throw new AbortedError();
    }
    if (!error?.response) {
      throw new NetworkError();
    }
    if (error.response.status >= 400 && error.response.status < 500) {
      throw new ClientError(error.config, error.request, error.response, getErrorMessage(error.response, "base.api.client_error"));
    }
    if (error.response.status >= 500 && error.response.status < 600) {
      throw new ServerError(error.config, error.request, error.response, getErrorMessage(error.response, "base.api.server_error"));
    }
    throw error;
  }
  return {
    name: "ErrorHandlerResponseInterceptor",
    type: "response" /* RESPONSE */,
    priority: -1e3,
    rejected,
    resolved
  };
}

// src/Plugin/Header/HeaderRequestInterceptor.ts
function HeaderRequestInterceptor(headers, method) {
  async function resolved(requestConfig) {
    if (method && requestConfig.method !== method) {
      return requestConfig;
    }
    Object.assign(requestConfig.headers, headers());
    return requestConfig;
  }
  return {
    name: "HeaderRequestInterceptor",
    type: "request" /* REQUEST */,
    priority: 1e3,
    resolved
  };
}

// src/Plugin/JwtBearerToken/JwtBearerTokenRequestInterceptor.ts
function JwtBearerTokenRequestInterceptor(tokenGetter, excludedRoutes = []) {
  async function resolved(config) {
    if (!config.url) {
      return config;
    }
    if (excludedRoutes.includes(config.url)) {
      return config;
    }
    config.headers.Authorization = `Bearer ${tokenGetter()}`;
    return config;
  }
  return {
    name: "JwtBearerTokenRequestInterceptor",
    type: "request" /* REQUEST */,
    priority: 100,
    resolved
  };
}

// src/HttpStatusCode.ts
var HttpStatusCode = /* @__PURE__ */ ((HttpStatusCode2) => {
  HttpStatusCode2[HttpStatusCode2["CONTINUE"] = 100] = "CONTINUE";
  HttpStatusCode2[HttpStatusCode2["SWITCHING_PROTOCOLS"] = 101] = "SWITCHING_PROTOCOLS";
  HttpStatusCode2[HttpStatusCode2["PROCESSING"] = 102] = "PROCESSING";
  HttpStatusCode2[HttpStatusCode2["EARLY_HINTS"] = 103] = "EARLY_HINTS";
  HttpStatusCode2[HttpStatusCode2["OK"] = 200] = "OK";
  HttpStatusCode2[HttpStatusCode2["CREATED"] = 201] = "CREATED";
  HttpStatusCode2[HttpStatusCode2["ACCEPTED"] = 202] = "ACCEPTED";
  HttpStatusCode2[HttpStatusCode2["NON_AUTHORITATIVE_INFORMATION"] = 203] = "NON_AUTHORITATIVE_INFORMATION";
  HttpStatusCode2[HttpStatusCode2["NO_CONTENT"] = 204] = "NO_CONTENT";
  HttpStatusCode2[HttpStatusCode2["RESET_CONTENT"] = 205] = "RESET_CONTENT";
  HttpStatusCode2[HttpStatusCode2["PARTIAL_CONTENT"] = 206] = "PARTIAL_CONTENT";
  HttpStatusCode2[HttpStatusCode2["MULTI_STATUS"] = 207] = "MULTI_STATUS";
  HttpStatusCode2[HttpStatusCode2["ALREADY_REPORTED"] = 208] = "ALREADY_REPORTED";
  HttpStatusCode2[HttpStatusCode2["IM_USED"] = 226] = "IM_USED";
  HttpStatusCode2[HttpStatusCode2["MULTIPLE_CHOICES"] = 300] = "MULTIPLE_CHOICES";
  HttpStatusCode2[HttpStatusCode2["MOVED_PERMANENTLY"] = 301] = "MOVED_PERMANENTLY";
  HttpStatusCode2[HttpStatusCode2["FOUND"] = 302] = "FOUND";
  HttpStatusCode2[HttpStatusCode2["SEE_OTHER"] = 303] = "SEE_OTHER";
  HttpStatusCode2[HttpStatusCode2["NOT_MODIFIED"] = 304] = "NOT_MODIFIED";
  HttpStatusCode2[HttpStatusCode2["USE_PROXY"] = 305] = "USE_PROXY";
  HttpStatusCode2[HttpStatusCode2["RESERVED"] = 306] = "RESERVED";
  HttpStatusCode2[HttpStatusCode2["TEMPORARY_REDIRECT"] = 307] = "TEMPORARY_REDIRECT";
  HttpStatusCode2[HttpStatusCode2["PERMANENTLY_REDIRECT"] = 308] = "PERMANENTLY_REDIRECT";
  HttpStatusCode2[HttpStatusCode2["BAD_REQUEST"] = 400] = "BAD_REQUEST";
  HttpStatusCode2[HttpStatusCode2["UNAUTHORIZED"] = 401] = "UNAUTHORIZED";
  HttpStatusCode2[HttpStatusCode2["PAYMENT_REQUIRED"] = 402] = "PAYMENT_REQUIRED";
  HttpStatusCode2[HttpStatusCode2["FORBIDDEN"] = 403] = "FORBIDDEN";
  HttpStatusCode2[HttpStatusCode2["NOT_FOUND"] = 404] = "NOT_FOUND";
  HttpStatusCode2[HttpStatusCode2["METHOD_NOT_ALLOWED"] = 405] = "METHOD_NOT_ALLOWED";
  HttpStatusCode2[HttpStatusCode2["NOT_ACCEPTABLE"] = 406] = "NOT_ACCEPTABLE";
  HttpStatusCode2[HttpStatusCode2["PROXY_AUTHENTICATION_REQUIRED"] = 407] = "PROXY_AUTHENTICATION_REQUIRED";
  HttpStatusCode2[HttpStatusCode2["REQUEST_TIMEOUT"] = 408] = "REQUEST_TIMEOUT";
  HttpStatusCode2[HttpStatusCode2["CONFLICT"] = 409] = "CONFLICT";
  HttpStatusCode2[HttpStatusCode2["GONE"] = 410] = "GONE";
  HttpStatusCode2[HttpStatusCode2["LENGTH_REQUIRED"] = 411] = "LENGTH_REQUIRED";
  HttpStatusCode2[HttpStatusCode2["PRECONDITION_FAILED"] = 412] = "PRECONDITION_FAILED";
  HttpStatusCode2[HttpStatusCode2["REQUEST_ENTITY_TOO_LARGE"] = 413] = "REQUEST_ENTITY_TOO_LARGE";
  HttpStatusCode2[HttpStatusCode2["REQUEST_URI_TOO_LONG"] = 414] = "REQUEST_URI_TOO_LONG";
  HttpStatusCode2[HttpStatusCode2["UNSUPPORTED_MEDIA_TYPE"] = 415] = "UNSUPPORTED_MEDIA_TYPE";
  HttpStatusCode2[HttpStatusCode2["REQUESTED_RANGE_NOT_SATISFIABLE"] = 416] = "REQUESTED_RANGE_NOT_SATISFIABLE";
  HttpStatusCode2[HttpStatusCode2["EXPECTATION_FAILED"] = 417] = "EXPECTATION_FAILED";
  HttpStatusCode2[HttpStatusCode2["I_AM_A_TEAPOT"] = 418] = "I_AM_A_TEAPOT";
  HttpStatusCode2[HttpStatusCode2["MISDIRECTED_REQUEST"] = 421] = "MISDIRECTED_REQUEST";
  HttpStatusCode2[HttpStatusCode2["UNPROCESSABLE_ENTITY"] = 422] = "UNPROCESSABLE_ENTITY";
  HttpStatusCode2[HttpStatusCode2["LOCKED"] = 423] = "LOCKED";
  HttpStatusCode2[HttpStatusCode2["FAILED_DEPENDENCY"] = 424] = "FAILED_DEPENDENCY";
  HttpStatusCode2[HttpStatusCode2["TOO_EARLY"] = 425] = "TOO_EARLY";
  HttpStatusCode2[HttpStatusCode2["UPGRADE_REQUIRED"] = 426] = "UPGRADE_REQUIRED";
  HttpStatusCode2[HttpStatusCode2["PRECONDITION_REQUIRED"] = 428] = "PRECONDITION_REQUIRED";
  HttpStatusCode2[HttpStatusCode2["TOO_MANY_REQUESTS"] = 429] = "TOO_MANY_REQUESTS";
  HttpStatusCode2[HttpStatusCode2["REQUEST_HEADER_FIELDS_TOO_LARGE"] = 431] = "REQUEST_HEADER_FIELDS_TOO_LARGE";
  HttpStatusCode2[HttpStatusCode2["UNAVAILABLE_FOR_LEGAL_REASONS"] = 451] = "UNAVAILABLE_FOR_LEGAL_REASONS";
  HttpStatusCode2[HttpStatusCode2["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
  HttpStatusCode2[HttpStatusCode2["NOT_IMPLEMENTED"] = 501] = "NOT_IMPLEMENTED";
  HttpStatusCode2[HttpStatusCode2["BAD_GATEWAY"] = 502] = "BAD_GATEWAY";
  HttpStatusCode2[HttpStatusCode2["SERVICE_UNAVAILABLE"] = 503] = "SERVICE_UNAVAILABLE";
  HttpStatusCode2[HttpStatusCode2["GATEWAY_TIMEOUT"] = 504] = "GATEWAY_TIMEOUT";
  HttpStatusCode2[HttpStatusCode2["VERSION_NOT_SUPPORTED"] = 505] = "VERSION_NOT_SUPPORTED";
  HttpStatusCode2[HttpStatusCode2["VARIANT_ALSO_NEGOTIATES_EXPERIMENTAL"] = 506] = "VARIANT_ALSO_NEGOTIATES_EXPERIMENTAL";
  HttpStatusCode2[HttpStatusCode2["INSUFFICIENT_STORAGE"] = 507] = "INSUFFICIENT_STORAGE";
  HttpStatusCode2[HttpStatusCode2["LOOP_DETECTED"] = 508] = "LOOP_DETECTED";
  HttpStatusCode2[HttpStatusCode2["NOT_EXTENDED"] = 510] = "NOT_EXTENDED";
  HttpStatusCode2[HttpStatusCode2["NETWORK_AUTHENTICATION_REQUIRED"] = 511] = "NETWORK_AUTHENTICATION_REQUIRED";
  return HttpStatusCode2;
})(HttpStatusCode || {});

// src/Plugin/RefreshToken/RefreshTokenResponseInterceptor.ts
function RefreshTokenResponseInterceptor(axiosInstance, refreshToken, excludedRoutes = []) {
  async function rejected(failedRequest) {
    if (!(failedRequest instanceof ClientError)) {
      throw failedRequest;
    }
    const failedRequestConfig = failedRequest.config;
    if (!failedRequestConfig.url) {
      throw failedRequest;
    }
    if (failedRequest.response.status !== 401 /* UNAUTHORIZED */) {
      throw failedRequest;
    }
    if (excludedRoutes.includes(failedRequestConfig.url)) {
      throw failedRequest;
    }
    if (failedRequestConfig._retry === false) {
      throw failedRequest;
    }
    await refreshToken();
    return axiosInstance.request(failedRequest.config);
  }
  return {
    name: "RefreshTokenResponseInterceptor",
    type: "response" /* RESPONSE */,
    priority: 100,
    rejected
  };
}

// src/Plugin/RefreshToken/Error/ExpiredRefreshTokenError.ts
import { AxiosError as AxiosError6 } from "axios";
var ExpiredRefreshTokenError = class _ExpiredRefreshTokenError extends AxiosError6 {
  constructor() {
    const message = "request.expired_refresh_token";
    super(message);
    this.message = message;
    this.name = "ExpiredRefreshTokenError";
    Object.defineProperty(this, "isAxiosError", {
      value: true,
      writable: false,
      enumerable: false,
      configurable: false
    });
    if ("captureStackTrace" in Error && typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, _ExpiredRefreshTokenError);
    } else {
      this.stack = new Error(message).stack;
    }
  }
};

// src/api.ts
var InterceptorType = /* @__PURE__ */ ((InterceptorType2) => {
  InterceptorType2["REQUEST"] = "request";
  InterceptorType2["RESPONSE"] = "response";
  return InterceptorType2;
})(InterceptorType || {});
var defaultConfig = {
  enableErrorHandler: true,
  interceptors: [],
  axiosConfig: {
    transitional: {
      clarifyTimeoutError: true
    }
  }
};
function createApi(config = defaultConfig) {
  const api = axios.create(defaultConfig.axiosConfig);
  const interceptors = [...config.interceptors];
  if (config.enableErrorHandler) {
    interceptors.push(ErrorHandlerResponseInterceptor());
  }
  const baseUrl = config.baseURL;
  if (baseUrl !== void 0) {
    let baseUrlGetter;
    if (typeof baseUrl === "string") {
      baseUrlGetter = () => baseUrl;
    } else {
      baseUrlGetter = baseUrl;
    }
    interceptors.push(BaseUrlRequestInterceptor(baseUrlGetter));
  }
  function installInterceptor(p) {
    interceptors.push(p);
    installInterceptors();
  }
  function installInterceptors() {
    api.interceptors.request.clear();
    api.interceptors.response.clear();
    interceptors.sort((a, b) => a.priority - b.priority).forEach((config2) => {
      switch (config2.type) {
        case "request" /* REQUEST */:
          api.interceptors.request.use(config2.resolved, config2.rejected);
          break;
        case "response" /* RESPONSE */:
          api.interceptors.response.use(config2.resolved, config2.rejected);
          break;
      }
    });
  }
  installInterceptors();
  return {
    api,
    installInterceptor
  };
}
export {
  AbortedError,
  BaseUrlRequestInterceptor,
  ClientError,
  ErrorHandlerResponseInterceptor,
  ExpiredRefreshTokenError,
  HeaderRequestInterceptor,
  HttpStatusCode,
  InterceptorType,
  JwtBearerTokenRequestInterceptor,
  NetworkError,
  RefreshTokenResponseInterceptor,
  ServerError,
  TimeoutError,
  createApi
};
//# sourceMappingURL=index.mjs.map